#!/usr/bin/python

import random
import heapq

class Field:
    def __init__(self, dish):
        self.content=[]
        self.dish=dish

    def __str__(self):
        if self.content==[]:
            return ' '
        else:
            return str(self.content[-1])

    def __iter__(self):
        return iter(self.content)

    def neighbors(self):
        return [self.dish[i] for i in self.dish.adjacent(self.dish.getcoord(self))]
    
    def directions(self):
        return [n - self for n in self.neighbors()]


    def random_direction(self):
        return random.choice(self.directions())

    def updatefield(self, obj, field='default'):
        if field=='default':
            field=self

        try:
            obj.field=field
        except:
            pass

    def getcoord(self):
        return self.dish.getcoord(self)

    def __sub__(self, b):
        x1, y1=self.getcoord()
        x2, y2=b.getcoord()

        return (x1-x2, y1-y2)

    def __add__(self, d):
        x1, y1=self.getcoord()
        x2, y2=d

        return self.dish[(x1+x2, y1+y2)]

class Dish:
    def __init__(self, sizex, sizey):
        self.sizex, self.sizey=sizex, sizey
        self.fields=[[Field(self) for j in range(sizex)] for i in range(sizey)]

    def __str__(self):
        res='--'*(self.sizex+1)+"-\n"
        for row in reversed(self.fields):
            res+='| '+' '.join(map(str, row))+" |\n"

        res+='--'*(self.sizex+1)+"-"
        return res

    def __getitem__(self, coord):
        x,y=coord
        return self.fields[y][x]

    def __setitem__(self, coord,val):
		raise NotImplementedError

    def getcoord(self, field):
        for row in self.fields:
            try:
                x=row.index(field)
                y=self.fields.index(row)
                return (x,y)
            except ValueError:
                pass
        raise ValueError

    def adjacent(self, coord):
        return [(i+coord[0], j+coord[1]) for i in [-1,0,1] for j in [-1,0,1] if (i!=0 or j!=0) and 0<=(i+coord[0])<self.sizex and 0<=(j+coord[1])<self.sizey]

    def __iter__(self):
        for row in self.fields:
            for f in row:
                yield f

 
if __name__ == '__main__':
    d=Dish(10,5)
    print d
    d[(3,4)].content.append('D')
    print d

    print d[(1,2)].neighbors()
    print d[(1,2)].directions()

    print d[(0,0)].directions()

    print d[(1,2)]-d[(0,0)]
    print repr(d[(0,0)]+(1,1))
    print d[(3,3)]+(0,1)
