#!/usr/bin/python

import random
import heapq

class Field:
    def __init__(self, dish):
        self.content=[]
        self.dish=dish

    def __str__(self):
        if self.content==[]:
            return ' '
        else:
            return str(self.content[-1])

    def push(self, obj):
        self.content.append(obj)
        self.updatefield(obj)
        self.after_push()


    def after_push(self):
        try:
            obj=self.content[-1]
        except:
            return

        try:
            above=self.content[-2]
        except:
            above=None

        def disturb():
            above.disturb(obj)

        def settle():
            if above==None or not above.dead:
                obj.settle(above)
            else:
                self.after_push()


        if above!=None:
            self.dish.scheduler.queue(0, disturb, [obj, above])
        self.dish.scheduler.queue(0, settle, [obj])
        

    def pop(self):
        res=self.content.pop()
        self.updatefield(res,None)

        return res

    def remove(self, obj):
        self.content.remove(obj)
        self.updatefield(obj,None)

    def __iter__(self):
        return iter(self.content)

    def neighbors(self):
        return [self.dish[i] for i in self.dish.adjacent(self.dish.getcoord(self))]
    
    def directions(self):
        return [n - self for n in self.neighbors()]


    def random_direction(self):
        return random.choice(self.directions())

    def updatefield(self, obj, field='default'):
        if field=='default':
            field=self

        try:
            obj.field=field
        except:
            pass

    def getcoord(self):
        return self.dish.getcoord(self)

    def __sub__(self, b):
        x1, y1=self.getcoord()
        x2, y2=b.getcoord()

        return (x1-x2, y1-y2)

    def __add__(self, d):
        x1, y1=self.getcoord()
        x2, y2=d

        return self.dish[(x1+x2, y1+y2)]

class Bacteria:
    symbol='B'
    genus="Bacterium"

    cycle=5
    speed=10

    dead=False

    actions=()

    def __str__(self):
        return self.symbol

    def create_offspring(self):
        b1=self.clone()
        self.field.push(b1)

    def divide(self):
        self.create_offspring()
        self.field.dish.scheduler.queue(self.cycle, self.live, [self])
		
    def move(self, d=None):
        if d==None:
            d=self.field.random_direction()


        def finish_move():
            field=self.field
            field.remove(self)
            field=field+d
            field.push(self)

        self.field.dish.scheduler.queue(self.speed, finish_move, [self])

    def clone(self):
        res=self.__class__()
        return res

    def settle(self, over):
        pass

    def disturb(self, under):
        pass

    def live(self):
        r=random.random()
        acc=0

        for a in self.actions:
            p, name=a
            acc+=p
            if r<acc:
                getattr(self, name)()
                return

        self.field.dish.scheduler.queue(self.cycle, self.live, [self])


    def die(self):
        self.field.remove(self)
        self.dead=True

class BacillusMundanus(Bacteria):
    symbol='M'
    max_unsettled=3
    genus="Bacillus Mundanus"

    actions=((0.1, 'move'),
            (0.1, 'divide'))


    def __init__(self):
        self.unsettled=0


    def settle(self, over):
        if over!=None:
            if self.unsettled>=self.max_unsettled:
                self.die()
            else:
                self.move(self.field.random_direction())
                self.unsettled+=1
            return
        else:
            self.unsettled=0

        self.field.dish.scheduler.queue(self.cycle, self.live, [self])


class Dish:
    def __init__(self, sizex, sizey):
        self.sizex, self.sizey=sizex, sizey
        self.fields=[[Field(self) for j in range(sizex)] for i in range(sizey)]

    def __str__(self):
        res='--'*(self.sizex+1)+"-\n"
        for row in reversed(self.fields):
            res+='| '+' '.join(map(str, row))+" |\n"

        res+='--'*(self.sizex+1)+"-"
        return res

    def population(self):
        res={}
        for f in self:
            for b in f:
                try:
                    genus=b.genus

                    try:
                        res[genus]+=1
                    except:
                        res[genus]=1
                except:
                    pass
        return res

    def __getitem__(self, coord):
        x,y=coord
        return self.fields[y][x]

    def __setitem__(self, coord,val):
		raise NotImplementedError

    def getcoord(self, field):
        for row in self.fields:
            try:
                x=row.index(field)
                y=self.fields.index(row)
                return (x,y)
            except ValueError:
                pass
        raise ValueError

    def adjacent(self, coord):
        return [(i+coord[0], j+coord[1]) for i in [-1,0,1] for j in [-1,0,1] if (i!=0 or j!=0) and 0<=(i+coord[0])<self.sizex and 0<=(j+coord[1])<self.sizey]

    def __iter__(self):
        for row in self.fields:
            for f in row:
                yield f

class Scheduler:
    def __init__(self):
        self.q=[]
        self.time=0

    def queue(self, deltat, f, relevant):
        heapq.heappush(self.q, (self.time+deltat, f, relevant))

    def run_one(self):
        t, f, relevant=heapq.heappop(self.q)
        self.time=t

        for obj in relevant:
            if obj.dead:
                return

        f()

    def run(self, max_time=-1, after_step=lambda :None):
        while max_time<0 or self.time<=max_time:
            self.run_one()
            after_step()


if __name__ == '__main__':
    d=Dish(10,5)
    d.scheduler=Scheduler()
    b=BacillusMundanus()
    d[(3,4)].push(b)

    def printing_generator():
        last=0
        while True:
            if d.scheduler.time>last:
                print d, d.population(), d.scheduler.time
                last=d.scheduler.time
            yield


    d.scheduler.run(1000, printing_generator().next)
